package com.xxl.job.admin.core.thread;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.xxl.job.admin.core.cron.CronExpression;
import com.xxl.job.admin.core.model.XxlJobInfo;
import com.xxl.job.admin.core.scheduler.ScheduleTypeEnum;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JobScheduleHelper {
  private static Logger logger = LoggerFactory.getLogger(com.xxl.job.admin.core.thread.JobScheduleHelper.class);
  
  private static com.xxl.job.admin.core.thread.JobScheduleHelper instance = new com.xxl.job.admin.core.thread.JobScheduleHelper();
  
  public static final long PRE_READ_MS = 5000L;
  
  private Thread scheduleThread;
  
  private Thread ringThread;
  
  public static com.xxl.job.admin.core.thread.JobScheduleHelper getInstance() {
    return instance;
  }
  
  private volatile boolean scheduleThreadToStop = false;
  
  private volatile boolean ringThreadToStop = false;
  
  private static volatile Map<Integer, List<Integer>> ringData = new ConcurrentHashMap<>();
  
  public void start() {
    this.scheduleThread = new Thread((Runnable)new Object(this));
    this.scheduleThread.setDaemon(true);
    this.scheduleThread.setName("xxl-job, admin JobScheduleHelper#scheduleThread");
    this.scheduleThread.start();
    this.ringThread = new Thread((Runnable)new Object(this));
    this.ringThread.setDaemon(true);
    this.ringThread.setName("xxl-job, admin JobScheduleHelper#ringThread");
    this.ringThread.start();
  }
  
  private void refreshNextValidTime(XxlJobInfo jobInfo, Date fromTime) {
    try {
      Date nextValidTime = generateNextValidTime(jobInfo, fromTime);
      if (nextValidTime != null) {
        jobInfo.setTriggerStatus(-1);
        jobInfo.setTriggerLastTime(jobInfo.getTriggerNextTime());
        jobInfo.setTriggerNextTime(nextValidTime.getTime());
      } else {
        jobInfo.setTriggerStatus(0);
        jobInfo.setTriggerLastTime(0L);
        jobInfo.setTriggerNextTime(0L);
        logger.error(">>>>>>>>>>> xxl-job, refreshNextValidTime fail for job: jobId={}, scheduleType={}, scheduleConf={}", new Object[] { Integer.valueOf(jobInfo.getId()), jobInfo.getScheduleType(), jobInfo.getScheduleConf() });
      } 
    } catch (Throwable e) {
      jobInfo.setTriggerStatus(0);
      jobInfo.setTriggerLastTime(0L);
      jobInfo.setTriggerNextTime(0L);
      logger.error(">>>>>>>>>>> xxl-job, refreshNextValidTime error for job: jobId={}, scheduleType={}, scheduleConf={}", new Object[] { Integer.valueOf(jobInfo.getId()), jobInfo.getScheduleType(), jobInfo.getScheduleConf(), e });
    } 
  }
  
  private void pushTimeRing(int ringSecond, int jobId) {
    List<Integer> ringItemData = ringData.get(Integer.valueOf(ringSecond));
    if (ringItemData == null) {
      ringItemData = new ArrayList<>();
      ringData.put(Integer.valueOf(ringSecond), ringItemData);
    } 
    ringItemData.add(Integer.valueOf(jobId));
    logger.debug(">>>>>>>>>>> xxl-job, schedule push time-ring : " + ringSecond + " = " + Arrays.<List>asList(new List[] { ringItemData }));
  }
  
  public void toStop() {
    this.scheduleThreadToStop = true;
    try {
      TimeUnit.SECONDS.sleep(1L);
    } catch (Throwable e) {
      logger.error(e.getMessage(), e);
    } 
    if (this.scheduleThread.getState() != Thread.State.TERMINATED) {
      this.scheduleThread.interrupt();
      try {
        this.scheduleThread.join();
      } catch (Throwable e) {
        logger.error(e.getMessage(), e);
      } 
    } 
    boolean hasRingData = false;
    if (!ringData.isEmpty())
      for (Iterator<Integer> iterator = ringData.keySet().iterator(); iterator.hasNext(); ) {
        int second = ((Integer)iterator.next()).intValue();
        List<Integer> tmpData = ringData.get(Integer.valueOf(second));
        if (tmpData != null && tmpData.size() > 0) {
          hasRingData = true;
          break;
        } 
      }  
    if (hasRingData)
      try {
        TimeUnit.SECONDS.sleep(8L);
      } catch (Throwable e) {
        logger.error(e.getMessage(), e);
      }  
    this.ringThreadToStop = true;
    try {
      TimeUnit.SECONDS.sleep(1L);
    } catch (Throwable e) {
      logger.error(e.getMessage(), e);
    } 
    if (this.ringThread.getState() != Thread.State.TERMINATED) {
      this.ringThread.interrupt();
      try {
        this.ringThread.join();
      } catch (Throwable e) {
        logger.error(e.getMessage(), e);
      } 
    } 
    logger.info(">>>>>>>>>>> xxl-job, JobScheduleHelper stop");
  }
  
  public static Date generateNextValidTime(XxlJobInfo jobInfo, Date fromTime) throws Exception {
    ScheduleTypeEnum scheduleTypeEnum = ScheduleTypeEnum.match(jobInfo.getScheduleType(), null);
    if (ScheduleTypeEnum.CRON == scheduleTypeEnum) {
      Date nextValidTime = (new CronExpression(jobInfo.getScheduleConf())).getNextValidTimeAfter(fromTime);
      return nextValidTime;
    } 
    if (ScheduleTypeEnum.FIX_RATE == scheduleTypeEnum)
      return new Date(fromTime.getTime() + (Integer.valueOf(jobInfo.getScheduleConf()).intValue() * 1000)); 
    return null;
  }
}
